require File.join(File.dirname(__FILE__), '../..', "tomte_protocol_spec_helper")

describe "Broker Reconnection simple test" do

  #
  # Environment
  #

  include EventedSpec::AMQPSpec
  default_options AMQP_OPTS
  default_timeout 60

  amqp_before do
    @ep  = Tomte::Protocol::Endpoint.new(:enveloper =>
              Tomte::Protocol::Enveloper::Generic.new(:serialization => :bson))

    @client = @ep.dialog_client(:exchange => "tomte.tests.external.dialog",
                      :exchange_options => {:durable => false})

    @server = @ep.dialog_server(:queue_options => {:durable => false} )
    @server.append_callback(:entity_ready) do |s|
      s.declare_exchange("tomte.tests.external.dialog", :direct, :durable => false) {|ex|
        s.bind(ex, :routing_key => "messages.all")
      }
    end


  end

  def close_and_done
    @server.close { @client.close {  @ep.disconnect! { done }}}
  end

  def send_probes(count=20, delay=1, &when_done)
    cnt = 0
    t = Tomte::Timer.periodic(delay) do
      cnt += 1
      @client.call({:ping => cnt}, :routing_key => "messages.all") do |metadata, response|
        expect(response[:pong]).to eq cnt
        puts cnt
        if cnt >= count
          t.cancel
          when_done.call
        end
      end
    end
  end

  def request_user_action(&when_done)
    puts "***** disconnect the broker NOW **** "
    when_done.call
  end


  amqp_after do

  end

  it 'is delivered and delivered back' do

    @server.on_call do |metadata, payload|
      expect(payload).to have_key("ping")
      {"pong" => payload['ping']}
    end

    @ep.connect! do
      @server.connect! do |s|
        s.consume

        @client.connect! do |c|
          send_probes { request_user_action { send_probes { close_and_done }}}
        end
      end
    end
  end
end