require File.join(File.dirname(__FILE__), '../..', "tomte_protocol_spec_helper")

describe "A Dialog" do
  describe "with an async-processing server" do

    #
    # Environment
    #

    include EventedSpec::AMQPSpec
    default_options AMQP_OPTS
    default_timeout 5

    amqp_before do
      @ep  = Tomte::Protocol::Endpoint.new(:bus => :test)

      @client = Tomte::Protocol::Dialog::Client.new(@ep,
                               :exchange => "tomte.tests.integration.dialog",
                               :exchange_options => {:auto_delete => true})

      @server = Tomte::Protocol::Dialog::Server.new(@ep,
                              :queue_options => {:auto_delete => true}, :raw => true )
      @server.append_callback(:entity_ready) do |s|
        s.declare_exchange("tomte.tests.integration.dialog", :direct, :auto_delete => true) {|ex|
          s.bind(ex, :routing_key => "messages.all")
        }
      end


    end

    def close_and_done
      @server.close { @client.close {  @ep.disconnect! { done }}}
    end

    amqp_after do

    end

    it 'is delivered and delivered back, after an async processing' do

      @server.on_call do |request|

        expect(request.payload).to eq "ping"
        expect(request).to be_a_kind_of Tomte::Protocol::Dialog::Server::Request

        callback = proc { |r| request.return(r) }
        job = proc { sleep (1); 'pong' }

        @server.protected_defer(job, callback)
        throw :_async
      end

      @ep.connect! do
        @server.connect! do |s|
          s.consume

          @client.connect! do |c|
            delay(0.5) do
              c.call("ping", :routing_key => "messages.all") do |metadata, response|

                expect(response).to eq "pong"
                close_and_done
              end
            end
          end
        end
      end
    end

    it 'handles an exception' do

      @server.on_call do |request|

        expect(request.payload).to eq "ping"
        expect(request).to be_a_kind_of Tomte::Protocol::Dialog::Server::Request

        callback = proc { |r| request.return(r) }
        errback = proc { |r| request.error(r) }
        job = proc { sleep (1); foo }

        @server.protected_defer(job, callback, errback)
        throw :_async
      end

      @ep.connect! do
        @server.connect! do |s|
          s.consume

          @client.connect! do |c|
            delay(0.5) do
              c.call("ping", :routing_key => "messages.all") do |metadata, response|

                expect(response).to be_a_kind_of Tomte::Protocol::RemoteException
                close_and_done
              end
            end
          end
        end
      end
    end
  end
end

describe "A RELIABLE Dialog" do
  describe "with an async-processing server" do

    #
    # Environment
    #

    include EventedSpec::AMQPSpec
    default_options AMQP_OPTS
    default_timeout 5

    amqp_before do
      @ep  = Tomte::Protocol::Endpoint.new(:bus => :test)

      @client = Tomte::Protocol::Dialog::Client.new(@ep,
                               :exchange => "tomte.tests.integration.dialog",
                               :reliable => true,
                               :exchange_options => {:auto_delete => true})

      @server = Tomte::Protocol::Dialog::Server.new(@ep,
                              :queue_options => {:auto_delete => true}, :raw => true )
      @server.append_callback(:entity_ready) do |s|
        s.declare_exchange("tomte.tests.integration.dialog", :direct, :auto_delete => true) {|ex|
          s.bind(ex, :routing_key => "messages.all")
        }
      end


    end

    def close_and_done
      @server.close { @client.close {  @ep.disconnect! { done }}}
    end

    amqp_after do

    end

    it 'is delivered and delivered back, after an async processing' do

      @server.on_call do |request|

        expect(request.payload).to eq "ping"
        expect(request).to be_a_kind_of Tomte::Protocol::Dialog::Server::Request

        callback = proc { |r| request.return(r) }
        job = proc { sleep (1); 'pong' }

        @server.protected_defer(job, callback)
        throw :_async
      end

      @ep.connect! do
        @server.connect! do |s|
          s.consume

          @client.connect! do |c|
            delay(0.5) do
              c.call("ping", :routing_key => "messages.all") do |metadata, response|

                expect(response).to eq "pong"
                close_and_done
              end
            end
          end
        end
      end
    end
  end
end