require File.join(File.dirname(__FILE__), '../..', "tomte_protocol_spec_helper")

describe "A 'Hello, world'-like example with a non-empty message and some update informations" do

  #
  # Environment
  #

  include EventedSpec::AMQPSpec
  default_options AMQP_OPTS
  default_timeout 5

  amqp_before do
    @ep  = Tomte::Protocol::Endpoint.new(:bus => :test)

    @client = Tomte::Protocol::Dialog::Client.new(@ep,
                             :exchange => "tomte.tests.integration.dialog",
                             :exchange_options => {:auto_delete => true})

    @server = Tomte::Protocol::Dialog::Server.new(@ep,
                            :queue_options => {:auto_delete => true} )
    @server.append_callback(:entity_ready) do |s|
      s.declare_exchange("tomte.tests.integration.dialog", :direct, :auto_delete => true) {|ex|
        s.bind(ex, :routing_key => "messages.all")
      }
    end


  end

  def close_and_done
    @server.close { @client.close {  @ep.disconnect! { done }}}
  end

  amqp_after do

  end

  it 'should NOT deliver updates when the client does not request thetm' do

   @server.on_call do |request|


     delay(0.2) do
       request.update :update
     end

     delay(0.3) do
       request.update :more_update
     end

     delay(0.4) do
       expect(request.payload).to eq "ping"
       request.return "pong"
     end

     throw :_async
   end

   @ep.connect! do
     @server.connect! do |s|
       s.consume

       @client.connect! do |c|
         delay(0.5) do

           c.call("ping", :routing_key => "messages.all") do |metadata, response|

             expect(response).to eq "pong"
             close_and_done
           end
         end
       end
     end
   end
  end

  it 'should deliver updates if requested' do

    @server.on_call do |request|


      delay(0.2) do
        request.update :update
      end

      delay(0.3) do
        request.update :more_update
      end

      delay(0.4) do
        expect(request.payload).to eq "ping"
        request.return "pong"
      end

      throw :_async
    end

    updates = []

    @ep.connect! do
      @server.connect! do |s|
        s.consume

        @client.connect! do |c|
          delay(0.5) do

            upd_cb = lambda {|update | updates << update }

            c.call("ping", :routing_key => "messages.all", :updates => upd_cb) do |metadata, response|

              expect(response).to eq "pong"
              expect(updates).to include('update')
              expect(updates).to include('more_update')

              close_and_done
            end
          end
        end
      end
    end
  end
end