require File.join(File.dirname(__FILE__), '../..', "tomte_protocol_spec_helper")

describe "In a generic Endpoint" do
  describe "A 'Hello, world'-like example with a non-empty message" do

    #
    # Environment
    #

    include EventedSpec::AMQPSpec
    default_options AMQP_OPTS
    default_timeout 5

    amqp_before do
      @ep  = Tomte::Protocol::Endpoint.new(:bus => :test)
      @channel   = @ep.new_channel
      @channel.on_error do |ch, close|
        raise "Channel-level error!: #{close.inspect}"
      end

      @queue    = @channel.queue("", :auto_delete => true)
      @exchange = @channel.direct("tomte.tests.integration.direct.exchange", :auto_delete => true)

      @queue.bind(@exchange, :routing_key => "messages.all")
    end



    it "is delivered" do
      consumer1 = Tomte::Protocol::Async::Consumer.new(@channel, @queue).consume
      consumer1.on_delivery do |metadata, payload|
        done
      end

      EventMachine.add_timer(0.5) do
        @exchange.publish("Hello!", :routing_key => "messages.all")
      end
    end
  end



  describe "A 'Hello, world'-like example with a blank message" do

    #
    # Environment
    #

    include EventedSpec::AMQPSpec
    default_options AMQP_OPTS
    default_timeout 5

    amqp_before do
      @ep  = Tomte::Protocol::Endpoint.new(:bus => :test)
      @channel   = @ep.new_channel
      @channel.on_error do |ch, close|
        raise "Channel-level error!: #{close.inspect}"
      end

      @queue    = @channel.queue("", :auto_delete => true)
      @exchange = @channel.direct("tomte.tests.integration.direct.exchange", :auto_delete => true)

      @queue.bind(@exchange, :routing_key => "messages.all")
    end



    it "is delivered" do
      consumer1 = Tomte::Protocol::Async::Consumer.new(@channel, @queue).consume
      consumer1.on_delivery do |metadata, payload|
        done
      end

      EventMachine.add_timer(0.5) do
        @exchange.publish("", :routing_key => "messages.all")
      end
    end
  end
end