require File.join(File.dirname(__FILE__), '../..', "tomte_protocol_spec_helper")

describe "A Consumer Entity" do

  include EventedSpec::AMQPSpec
  default_options AMQP_OPTS
  default_timeout 5


  amqp_before do
    @ep = Tomte::Protocol::Endpoint.new(:bus => :test)
    @channel = @ep.new_channel

    @channel.on_error do |ch, close|
      raise "Channel-level error!: #{close.inspect}"
    end
  end

  def close_and_done

    @ep.disconnect! { done }
  end

  describe 'regarding queue declarations' do

    it 'should apply default options' do

      @consumer = @ep.consumer(:queue => "tomte.tests.integration.queue")

      @ep.connect! do
        @consumer.connect! do |p|

          @channel.queue("tomte.tests.integration.direct.queue",
                          Tomte::Protocol::Consumer::DEFAULT_QUEUE_OPTIONS.dup) do |exc|

            p.delete_queue { p.close { close_and_done } }
          end
        end
      end
    end

    it 'should apply passed options' do

      @consumer = @ep.consumer(:queue => "tomte.tests.integration.queue",
                               :queue_options => {:auto_delete => true})

      @ep.connect! do
        @consumer.connect! do |p|

          @channel.queue("tomte.tests.integration.queue",
                         :auto_delete => true) do |exc|

            p.delete_queue { p.close { close_and_done } }
          end
        end
      end
    end
  end


end