require File.join(File.dirname(__FILE__), '../..', "tomte_protocol_spec_helper")

describe "A Consumer" do
  describe "regarding on_delivery parameters" do

    #
    # Environment
    #

    include EventedSpec::AMQPSpec
    default_options AMQP_OPTS
    default_timeout 5

    amqp_before do
      @ep  = Tomte::Protocol::Endpoint.new(:bus => :test)

      @producer = @ep.producer(:exchange => "tomte.tests.integration.direct.exchange",
                               :exchange_options => {:auto_delete => true})

      @consumer = @ep.consumer(:queue_options => {:auto_delete => true})
    end


    def bind_and_publish

      @ep.connect! do
        @producer.connect! do |p|
          @consumer.connect! do |c|
            c.bind("tomte.tests.integration.direct.exchange",
                    :routing_key => "messages.all")

            c.consume
          end

          EventMachine.add_timer(0.5) do
            p.publish("Hello!", :routing_key => "messages.all")
          end
        end
      end
    end

    def close_and_done
      @producer.close { @consumer.close {  @ep.disconnect! { done }}}
    end

    amqp_after do

    end

    it "with one parameter" do

      @consumer.on_delivery do |payload|

        expect(payload).to eq "Hello!"
        close_and_done
      end

      bind_and_publish
    end

    it "with two parameters" do

      @consumer.on_delivery do |metadata,payload|

        expect(payload).to eq "Hello!"
        expect(metadata).to be_a_kind_of AMQP::Header
        close_and_done
      end

      bind_and_publish
    end

    it "with seven parameters" do

      @consumer.on_delivery do |metadata, payload, consumer_tag, delivery_tag, redelivered, exchange, routing_key|

        expect(payload).to eq "Hello!"
        expect(metadata).to be_a_kind_of AMQP::Header
        close_and_done
      end

      bind_and_publish
    end
  end
end