require File.join(File.dirname(__FILE__), '../..', "tomte_protocol_spec_helper")

describe "A Producer Entity" do

  include EventedSpec::AMQPSpec
  default_options AMQP_OPTS
  default_timeout 5


  amqp_before do
    @ep = Tomte::Protocol::Endpoint.new(:bus => :test)
    @channel = @ep.new_channel

    @channel.on_error do |ch, close|
      raise "Channel-level error!: #{close.inspect}"
    end
  end

  def close_and_done

    @ep.disconnect! { done }
  end

  describe 'regarding exchange declarations' do

    it 'should apply default options' do

      @producer = @ep.producer(:exchange => "tomte.tests.integration.direct.exchange")

      @ep.connect! do
        @producer.connect! do |p|

          @channel.direct("tomte.tests.integration.direct.exchange",
                          Tomte::Protocol::Producer::DEFAULT_EXCHANGE_OPTIONS.dup) do |exc|

            p.delete_exchange { p.close { close_and_done } }
          end
        end
      end
    end

    it 'should apply passed options' do

      @producer = @ep.producer(:exchange => "tomte.tests.integration.direct.exchange",
                               :exchange_options => {:auto_delete => true})

      @ep.connect! do
        @producer.connect! do |p|

          @channel.direct("tomte.tests.integration.direct.exchange",
                          :auto_delete => true) do |exc|

            p.delete_exchange { p.close { close_and_done } }
          end
        end
      end
    end

    it 'should apply the relevant type (try fanout)' do

      @producer = @ep.producer(:exchange => "tomte.tests.integration.fanout.exchange",
                               :exchange_type => :fanout,
                               :exchange_options => {:auto_delete => true})

      @ep.connect! do
        @producer.connect! do |p|
          @channel.fanout("tomte.tests.integration.fanout.exchange",
                                   :auto_delete => true) do |exc|

            p.delete_exchange { p.close { close_and_done } }
          end
        end
      end
    end

    it 'should apply the relevant type (try topic)' do

      @producer = @ep.producer(:exchange => "tomte.tests.integration.topic.exchange",
                               :exchange_type => :topic,
                               :exchange_options => {:auto_delete => true})

      @ep.connect! do
        @producer.connect! do |p|
          @channel.topic("tomte.tests.integration.topic.exchange",
                                   :auto_delete => true) do |exc|

            p.delete_exchange { p.close { close_and_done } }
          end

        end
      end
    end
  end


end