require File.join(File.dirname(__FILE__), '../..', "tomte_protocol_spec_helper")

describe "A Producer Entity" do

  include EventedSpec::AMQPSpec
  default_options AMQP_OPTS
  default_timeout 5


  amqp_before do
    @ep = Tomte::Protocol::Endpoint.new(:bus => :test)
    @channel = @ep.new_channel

    @channel.on_error do |ch, close|
      raise "Channel-level error!: #{close.inspect}"
    end
  end

  def close_and_done

    @channel.close { @ep.disconnect! { done } }
  end

  describe 'regarding errors' do

    it 'should call the errback in case of failure declaring the exchange' do

      @channel.fanout("tomte.tests.integration.fanout.exchange", :auto_delete => true) do

        @producer = @ep.producer(:exchange => "tomte.tests.integration.fanout.exchange",
                                 :exchange_options => {:auto_delete => true})

        errback = proc { close_and_done }
        @ep.connect! do
          @producer.connect!(:errback => errback) do
            fail
          end
        end
      end
    end

    it 'should call the errback in case of a publish error' do

      @producer = @ep.producer(:exchange => "tomte.tests.integration.direct.exchange",
                               :exchange_options => {:auto_delete => true})

      errback = proc { |e|  expect(e).to be_a_kind_of Tomte::Protocol::FormatException
                            close_and_done }

      class DummyClass; end

      @producer.enveloper = Tomte::Protocol::Enveloper::Generic.new

      @ep.connect! do
        @producer.connect! do |p|

          p.publish(DummyClass.new, :errback => errback) do |res|
            fail "should not be called"
          end
        end
      end
    end
  end
end