require File.join(File.dirname(__FILE__), '../..', "tomte_protocol_spec_helper")
require 'tomte/protocol/enveloper'
require 'tomte/message'

class MyDoc
  include Tomte::Document

  key :foo, String
  key :bar, String, :default => "barz"

end

class MyInheritedDoc < MyDoc

  key :baz, String

end

describe "Going through a generic enveloper" do


  include EventedSpec::AMQPSpec
  default_options AMQP_OPTS
  default_timeout 5

  amqp_before do

    @doc = MyDoc.new(:foo => "fooz")

    @msg = Tomte::Message.new(
        :metadata => Tomte::Message::Metadata.new(:message_id => "1234"),
        :payload => @doc,
        :headers => {:a => 42, :b => 23}
    )

    @ep  = Tomte::Protocol::Endpoint.new(:bus => :test)
    @ep.enveloper = Tomte::Protocol::Enveloper::Generic.new(:serialization => :bson)

    @producer = @ep.producer(:exchange => "tomte.tests.integration.direct.exchange",
                             :exchange_options => {:auto_delete => true})

    @consumer = @ep.consumer(:queue_options => {:auto_delete => true})

  end

  def close_and_done
    @producer.close { @consumer.close {  @ep.disconnect! { done }}}
  end

  it "A Message is delivered with all the attributes" do

    @consumer.on_delivery do |metadata, payload|

      expect(payload).to be_a_kind_of Tomte::Document
      expect(payload.foo).to eq "fooz"
      expect(payload.bar).to eq "barz"

      expect(metadata.headers['a']).to eq 42
      expect(metadata.message_id).to eq "1234"

      close_and_done
    end

    @producer.connect! do |p|
      @consumer.connect! do |c|
        c.bind("tomte.tests.integration.direct.exchange",
                :routing_key => "messages.all")

        c.consume
      end

      delay(0.5) do
        p.publish(@msg, :routing_key => "messages.all")
      end
    end
  end

  it "A Document is delivered with all the attributes" do

    @consumer.on_delivery do |metadata, payload|

      expect(payload).to be_a_kind_of Tomte::Document
      expect(payload.foo).to eq "fooz"
      expect(payload.bar).to eq "barz"

      expect(metadata.headers['a']).to eq 42
      expect(metadata.message_id).to eq "1234"

      close_and_done
    end

    @producer.connect! do |p|
      @consumer.connect! do |c|
        c.bind("tomte.tests.integration.direct.exchange",
                :routing_key => "messages.all")

        c.consume
      end

      delay(0.5) do
        p.publish(@doc,
                  :headers => {:a => 42, :b => 23},
                  :message_id => "1234",
                  :routing_key => "messages.all")
      end
    end
  end

  it "An inherited Document is delivered with all the attributes" do

    doc = MyInheritedDoc.new(:foo => "fooz", :baz => "bazz")

    @consumer.on_delivery do |metadata, payload|

      expect(payload).to be_a_kind_of Tomte::Document
      expect(payload.foo).to eq "fooz"
      expect(payload.bar).to eq "barz"
      expect(payload.baz).to eq "bazz"

      expect(metadata.headers['a']).to eq 42
      expect(metadata.message_id).to eq "1234"

      close_and_done
    end

    @producer.connect! do |p|
      @consumer.connect! do |c|
        c.bind("tomte.tests.integration.direct.exchange",
                :routing_key => "messages.all")

        c.consume
      end

      delay(0.5) do
        p.publish(doc,
                  :headers => {:a => 42, :b => 23},
                  :message_id => "1234",
                  :routing_key => "messages.all")
      end
    end
  end

  it "A Hash is delivered with all the attributes" do

    @consumer.on_delivery do |metadata, payload|

      expect(payload).to be_a_kind_of Hash
      expect(payload['a']).to eq 1
      expect(payload['b']).to eq 2

      expect(metadata.headers['a']).to eq 42
      expect(metadata.message_id).to eq "1234"

      close_and_done
    end

    @producer.connect! do |p|
      @consumer.connect! do |c|
        c.bind("tomte.tests.integration.direct.exchange",
                :routing_key => "messages.all")

        c.consume
      end

      delay(0.5) do
        p.publish({:a => 1, :b => 2},
                  :headers => {:a => 42, :b => 23},
                  :message_id => "1234",
                  :routing_key => "messages.all")
      end
    end
  end
end