require File.join(File.dirname(__FILE__), '../..', "tomte_protocol_spec_helper")
require 'tomte/protocol/enveloper'
require 'tomte/message'

describe "Going through a null enveloper" do


  include EventedSpec::AMQPSpec
  default_options AMQP_OPTS
  default_timeout 5

  amqp_before do

    @msg = "foobar"

    @ep  = Tomte::Protocol::Endpoint.new(:bus => :test)
    @ep.enveloper = Tomte::Protocol::Enveloper::Null.new

    @producer = @ep.producer(:exchange => "tomte.tests.integration.direct.exchange",
                             :exchange_options => {:auto_delete => true})

    @consumer = @ep.consumer(:queue_options => {:auto_delete => true})

  end

  def close_and_done
    @producer.close { @consumer.close {  @ep.disconnect! { done }}}
  end

  it "A String is delivered as string" do

    @consumer.on_delivery do |metadata, payload|

      expect(payload).to be_a_kind_of(String)
      expect(payload).to eq @msg

      close_and_done
    end

    @producer.connect! do |p|
      @consumer.connect! do |c|
        c.bind("tomte.tests.integration.direct.exchange",
                :routing_key => "messages.all")

        c.consume
      end

      delay(0.5) do
        p.publish(@msg, :routing_key => "messages.all")
      end
    end
  end

  it "headers are set anyways" do

    @consumer.on_delivery do |metadata, payload|

      expect(payload).to be_a_kind_of(String)
      expect(payload).to eq @msg

      expect(metadata.headers['a']).to eq 42
      expect(metadata.message_id).to eq "1234"

      close_and_done
    end

    @producer.connect! do |p|
      @consumer.connect! do |c|
        c.bind("tomte.tests.integration.direct.exchange",
                :routing_key => "messages.all")

        c.consume
      end

      delay(0.5) do
        p.publish(@msg,
                  :headers => {:a => 42, :b => 23},
                  :message_id => "1234",
                  :routing_key => "messages.all")
      end
    end
  end
end