require File.join(File.dirname(__FILE__), '../..', "tomte_protocol_spec_helper")

describe "A RPC pair" do
  describe "with an async-processing server" do

    #
    # Environment
    #

    include EventedSpec::AMQPSpec
    default_options AMQP_OPTS
    default_timeout 5

    amqp_before do
      @ep  = Tomte::Protocol::Endpoint.new(:bus => :test)

      @client = Tomte::Protocol::RPC::Client.new(@ep,
                               :api => "tomte.tests.integration.rpc",
                               :exchange_options => {:auto_delete => true})

      @server = Tomte::Protocol::RPC::Server.new(@ep,
                              :api =>"tomte.tests.integration.rpc",
                              :exchange_options => {:auto_delete => true},
                              :queue_options => {:auto_delete => true}, :raw => true )
    end



    def close_and_done
      @server.close { @client.close {  @ep.disconnect! { done }}}
    end

    amqp_after do

    end

    it 'is delivered and delivered back after an async processing' do

      class AsyncServer

        def ping(request)
          callback = proc { |r| request.return(r) }
          job = proc { sleep (1); 'pong' }

          Tomte::Protocol::RPC::Server.protected_defer(job, callback)
          throw :_async
        end

      end

      @server.server= AsyncServer.new

      @ep.connect! do
        @server.ready! do |s|

          @client.connect! do |c|
            delay(0.5) do
              c.call("ping") do |response|

                expect(response).to eq 'pong'
                close_and_done
              end
            end
          end
        end
      end
    end

    it 'handles exception happening during the processing (raw)' do

      class AsyncServer

        def ping(request)
          callback = proc { |r| request.return(r) }
          errback = proc { |r| request.error(r) }
          job = proc { sleep (1); foo }

          Tomte::Protocol::RPC::Server.protected_defer(job, callback, errback)
          throw :_async
        end

      end

      @server.server= AsyncServer.new

      @ep.connect! do
        @server.ready! do |s|

          @client.connect! do |c|
            delay(0.5) do
              c.call('ping') do |response|
                expect(response).to be_a_kind_of Tomte::Protocol::RPCRemoteException
                close_and_done
              end
            end
          end
        end
      end
    end
  end
end