require File.join(File.dirname(__FILE__), '../..', "tomte_protocol_spec_helper")

describe "A RPC pair" do
  describe "With an exception at the server Side" do

    #
    # Environment
    #

    include EventedSpec::AMQPSpec
    default_options AMQP_OPTS
    default_timeout 5

    amqp_before do
      @ep  = Tomte::Protocol::Endpoint.new(:bus => :test)

      @client = Tomte::Protocol::RPC::Client.new(@ep,
                               :api => "tomte.tests.integration.rpc",
                               :exchange_options => {:auto_delete => true})

      @server = Tomte::Protocol::RPC::Server.new(@ep,
                              :api =>"tomte.tests.integration.rpc",
                              :exchange_options => {:auto_delete => true},
                              :queue_options => {:auto_delete => true} )
    end



    def close_and_done
      @server.close { @client.close {  @ep.disconnect! { done }}}
    end

    amqp_after do

    end

    it 'is delivered and delivered back with an exception' do

      class Server
        def ping
          raise ArgumentError, "this is an error"
        end
      end

      @server.server= Server.new

      @ep.connect! do
        @server.ready! do |s|

          @client.connect! do |c|
            delay(0.5) do
              c.call("ping") do |response|

                expect(response).to be_a_kind_of Tomte::Protocol::RPCRemoteException
                expect(response.remote).to be_a_kind_of ArgumentError
                close_and_done
              end
            end
          end
        end
      end
    end


    it 'handles correctly a publish error' do

      class DummyClass; end

      @client.enveloper = Tomte::Protocol::Enveloper::Generic.new
      @client.connect! do |c|
        delay(0.5) do
          c.call(:foo, :args => [DummyClass.new]) do |response|
            expect(response).to be_a_kind_of Tomte::Protocol::FormatException
            close_and_done
          end
        end
      end
    end
  end
end