require File.join(File.dirname(__FILE__), '../..', "tomte_protocol_spec_helper")

describe "A RPC pair" do
  describe "With a class as a server" do

    #
    # Environment
    #

    include EventedSpec::AMQPSpec
    default_options AMQP_OPTS
    default_timeout 5

    class TestServer
        def ping
          :pong
        end

        def sum(a, b)
          a+b
        end

        def increment(a)
          a +=1
        end

        def hashy(args = {})

          return args[:foo] + args[:bar]
        end

        def error_method(a)
          poop
        end
      end

    amqp_before do
      @ep  = Tomte::Protocol::Endpoint.new(:bus => :test)

      @client = Tomte::Protocol::RPC::Client.new(@ep,
                               :api => "tomte.tests.integration.rpc",
                               :exchange_options => {:auto_delete => true})

      @server = Tomte::Protocol::RPC::Server.new(@ep,
                              :api =>"tomte.tests.integration.rpc",
                              :exchange_options => {:auto_delete => true},
                              :queue_options => {:auto_delete => true} )

      @server.server= TestServer.new
    end



    def close_and_done
      @server.close { @client.close {  @ep.disconnect! { done }}}
    end

    amqp_after do

    end

    it 'works with no parameters' do

      @ep.connect! do
        @server.ready! do |s|

          @client.connect! do |c|
            delay(0.5) do
              c.proxy.ping do |response|
                expect(response).to eq :pong
                close_and_done
              end
            end
          end
        end
      end
    end

    it 'works with one parameter' do

      @ep.connect! do
        @server.ready! do |s|

          @client.connect! do |c|
            delay(0.5) do
              c.proxy.increment(42) do |response|
                expect(response).to eq 43
                close_and_done
              end
            end
          end
        end
      end
    end

    it 'works with more parameters' do

      @ep.connect! do
        @server.ready! do |s|

          @client.connect! do |c|
            delay(0.5) do
              c.proxy.sum(3,5) do |response|
                expect(response).to eq 8
                close_and_done
              end
            end
          end
        end
      end
    end

    it 'raises an exception when the method count is wrong' do

      @ep.connect! do
        @server.ready! do |s|

          @client.connect! do |c|
            delay(0.5) do
              c.proxy.error_method do |response|

                expect(response).to be_a_kind_of Tomte::Protocol::RPCRemoteException
                expect(response.remote).to be_a_kind_of ArgumentError

                close_and_done
              end
            end
          end
        end
      end
    end

    it 'raises an exception when the method is not found' do

      @ep.connect! do
        @server.ready! do |s|

          @client.connect! do |c|
            delay(0.5) do
              c.proxy.no_method(:foo) do |response|

                expect(response).to be_a_kind_of Tomte::Protocol::RPCRemoteException
                expect(response.remote).to be_a_kind_of NoMethodError

                close_and_done
              end
            end
          end
        end
      end
    end

  end
end