require File.join(File.dirname(__FILE__), '../..', "tomte_protocol_spec_helper")

describe "A RPC pair" do
  describe "With a class as a server" do

    #
    # Environment
    #

    include EventedSpec::AMQPSpec
    default_options AMQP_OPTS
    default_timeout 5

    class TestServer
        def ping
          :pong
        end

        def sum(a, b)
          a+b
        end

        def increment(a)
          a +=1
        end

        def hashy(args = {})

          return args[:foo] + args[:bar]
        end

        def hashy_response(args  = {})

          args.each { |k, v| args[k] +=1 }
          return args
        end

        def error_method(a)
          poop
        end
      end

    amqp_before do
      @ep  = Tomte::Protocol::Endpoint.new(:bus => :test)

      @client = Tomte::Protocol::RPC::Client.new(@ep,
                               :api => "tomte.tests.integration.rpc",
                               :exchange_options => {:auto_delete => true})

      @server = Tomte::Protocol::RPC::Server.new(@ep,
                              :api =>"tomte.tests.integration.rpc",
                              :exchange_options => {:auto_delete => true},
                              :queue_options => {:auto_delete => true} )

      @server.server= TestServer.new
    end



    def close_and_done
      @server.close { @client.close {  @ep.disconnect! { done }}}
    end

    amqp_after do

    end



    it 'works with a hash too' do

      @ep.connect! do
        @server.ready! do |s|

          @client.connect! do |c|
            delay(0.5) do
              c.proxy.hashy(:foo => 3, :bar => 8) do |response|
                expect(response).to eq 11
                close_and_done
              end
            end
          end
        end
      end
    end

    it 'works with returning a hash too' do

      @ep.connect! do
        @server.ready! do |s|

          @client.connect! do |c|
            delay(0.5) do
              c.proxy.hashy_response(:foo => 3, :bar => 8) do |response|
                expect(response).to be_a_kind_of Hash
                expect(response[:foo]).to eq 4
                expect(response[:bar]).to eq 9

                close_and_done
              end
            end
          end
        end
      end
    end

  end
end