require File.join(File.dirname(__FILE__), '../..', "tomte_protocol_spec_helper")


describe "A Synchronized" do

  describe "unreliable Producer" do

    before :all do
      @ep = Tomte::Protocol::Endpoint.new(:sync => true, :bus => :test).connect!
      expect(@ep.client.status).to eq :opened

      @producer = Tomte::Protocol::Sync::Producer.new(@ep, :reliable => false)
      @producer.connect!
    end

    after :all do
      @producer.close
      @ep.disconnect!
    end


    it 'should connect' do
      expect(@producer).to be_connected
    end

    it 'should publish' do
      expect(@producer).to be_connected
      res = @producer.publish("hello")
      expect(res).to be_nil
    end

    it 'should publish even if unroutable' do
      expect(@producer).to be_connected
      res = @producer.publish("hello", :mandatory => true)
      expect(res).to be_nil
    end
  end

  describe "reliable Producer" do

    before :all do
      @ep = Tomte::Protocol::Endpoint.new(:sync => true, :bus => :test).connect!
      expect(@ep.client.status).to eq :opened

      @producer = Tomte::Protocol::Sync::Producer.new(@ep, :reliable => true)
      @producer.connect!
    end

    after :all do
      @producer.close
      @ep.disconnect!
    end


    it 'should connect' do
      expect(@producer).to be_connected
    end

    it 'should publish' do
      expect(@producer).to be_connected
      res = @producer.publish("hello", :mandatory => false, :immediate => false )
      expect(res).to be_nil
    end

    it 'should fail if unroutable' do
      expect(@producer).to be_connected

      expect { @producer.publish("hello", :mandatory => true)}.to raise_error

    end
  end

end


