require File.join(File.dirname(__FILE__), '../../..', "tomte_protocol_spec_helper")


describe "A Synchronized" do
  describe "RPC" do

    def endpoint
      @ep ||= Tomte::Protocol::Endpoint.new(:sync => true, :bus => :test).connect!
    end

    before :each do

    end

    after :each do
      endpoint.disconnect!
    end


    it 'should transport en exception, and raise it on return' do

      server = endpoint.rpc_server(:api => 'tomte.tests.integration.rpc').connect!
      server.server do |verb, request|
        expect(verb).to eq :ping

        raise ArgumentError, "hello"
      end
      server.consume

      client = endpoint.rpc_client(:api => 'tomte.tests.integration.rpc').connect!

      expect { res = client.call(:ping) }.to raise_error Tomte::Protocol::RPCRemoteException
      server.close
      client.close
    end

    it 'should transport en exception, containing the original exception' do

      server = endpoint.rpc_server(:api => 'tomte.tests.integration.rpc').connect!
      server.server do |verb, request|
        expect(verb).to eq :ping

        raise ArgumentError, "hello"
      end
      server.consume

      client = endpoint.rpc_client(:api => 'tomte.tests.integration.rpc').connect!

      begin
        res = client.call(:ping)
      rescue Tomte::Protocol::RPCRemoteException => e
        expect(e.remote).to be_a_kind_of ArgumentError
        res = :done
      end

      expect(res).to eq :done
      server.close
      client.close
    end


    it 'should transport en exception, and raise it on return (proxy)' do

      server = endpoint.rpc_server(:api => 'tomte.tests.integration.rpc').connect!
      server.server do |verb, request|
        expect(verb).to eq :ping

        raise ArgumentError, "hello"
      end
      server.consume

      client = endpoint.rpc_client(:api => 'tomte.tests.integration.rpc').connect!
      proxy = client.proxy

      expect { res = proxy.ping }.to raise_error ArgumentError

      server.close
      client.close
    end



    it 'should behave good in a serializer exception' do
      server = endpoint.rpc_server(:api => 'tomte.tests.integration.rpc').connect!
      server.server do |verb, request|

        raise ArgumentError, "hello"
      end
      server.consume

      client = endpoint.rpc_client(:api => 'tomte.tests.integration.rpc').connect!
      proxy = client.proxy

      class DummyClass; end
      expect {res = proxy.ping(DummyClass.new) }.to raise_error Tomte::Protocol::FormatException
    end

  end
end