require File.join(File.dirname(__FILE__), '../..', "tomte_protocol_spec_helper")
require 'tomte/protocol/serializer'

def sample_object
  HashWithIndifferentAccess.new({:a => 1, :b => [1, 2, 3], :c => "hello"})
end

def sample_json
  '{"a":1,"b":[1,2,3],"c":"hello"}'
end

def sample_bson
  ::BSON.serialize(sample_object)
end

describe "the Serializer module" do

  S = Tomte::Protocol::Serializer

  it 'should determine the content-type for the basic formats' do
    TYPES = {
      :text => 'text/plain',
      :json => 'application/json',
      :bson => 'application/x-tomte-bson',
    }.freeze

    TYPES.each do |format, content_type|
      expect(S.content_type_for(format)).to eq content_type
    end
  end

  it 'should determine the format for the basic content type' do
    FORMATS = {
        :text => ['text/plain'],
        :json => ['application/json', 'application/x-json'],
        :bson => ['application/x-tomte-bson', 'application/x-bson'],
    }.freeze
    FORMATS.each do |format, content_types|
      content_types.each do |type|
        expect(S.format_for(type)).to eq format
      end
    end
  end


  it 'should raise an error if format is not found' do
    expect { S.content_type_for(:bloop) }.to raise_error Tomte::Protocol::Serializer::FormatError
  end

  it 'should raise an error if type is not found' do
    expect { S.format_for("fake/x-type") }.to raise_error Tomte::Protocol::Serializer::FormatError
  end

  it 'should be able to encode json' do
    S.encode(sample_object, :json)
  end

  it 'should be able to decode json' do
    S.decode(sample_json, :json)
    S.decode_with_type(sample_json, 'application/json')
  end

  it 'should be able to encode/decode json' do
    expect(HashWithIndifferentAccess.new(S.decode(S.encode(sample_object, :json), :json))).to eq sample_object
  end

  it 'should be able to encode bson' do
     S.encode(sample_object, :bson)
  end

  it 'should be able to decode bson' do
    S.decode(sample_bson, :bson)
    S.decode_with_type(sample_bson, 'application/x-tomte-bson')
  end

  it 'should be able to encode/decode bson' do
    expect(HashWithIndifferentAccess.new(S.decode(S.encode(sample_object, :bson), :bson))).to eq sample_object
  end

end