require File.join(File.dirname(__FILE__), '../..', "tomte_protocol_spec_helper")
require 'tomte/protocol/enveloper'
require 'tomte/message'

class EnvelopeTest
  include Tomte::Protocol::Enveloper::Hooks

  def initialize(enveloper)
    @enveloper = enveloper
  end

end

def test_message
  Tomte::Message.new do |m|
    m.define_header do |h|
      h.header_foo = :foo
      h.header_bar = :bar
    end
    m.define_metadata do |a|
      a.md_foo = :foo
      a.md_bar = :bar
    end
    m.foo = :foo
    m.bar = :bar
  end
end

def test_text
%Q{
--- !ruby/object:Tomte::Message
table:
  :header: !ruby/object:Tomte::Message::Header
    table:
      :header_foo: :foo
      :header_bar: :bar
      :local_time: "2010-03-22T15:53:01+01:00"
      :uuid: cade2f68-6316-4361-a3a8-cab74e507112
      :host: foobar
      :identity: tomte-test-foobar
  :foo: :testingtest
  :bar: :bar
}
end

describe "A Test Class with an enveloper" do


  before :each do
    @e = Tomte::Protocol::Enveloper::Generic.new
    @t = EnvelopeTest.new(@e)
  end


  it 'should be able to envelope a primitive object' do

    h = {:a => :a, :b => :b}
    res = @t.envelope(h)
    expect(res).to be_a_kind_of String
    expect(res).not_to be_empty

  end

  it 'should be able to envelope a T::M' do
    m = test_message
    res = @t.envelope(m)
    expect(res).to be_a_kind_of String
    expect(res).not_to be_empty
  end

  it 'should be reversable for primitive objects' do
    h = {:a => :a, :b => :b}
    opts = {}
    res = @t.envelope(h, opts)
    orig = @t.de_envelope(res, opts)
    expect(orig).to be_a_kind_of Hash
    expect(orig).to eq  h
  end


  it 'should be reversabe for messages' do
    m = test_message
    opts = {}
    res = @t.envelope(m.dup, opts)
    orig = @t.de_envelope(res, opts)
    expect(orig).to be_a_kind_of Tomte::Message
    expect(orig.foo).to eq m.foo
  end

  it 'should correctly deserialize messages' do
    orig = @t.de_envelope(test_text, {:content_type => 'application/x-yaml'})
    expect(orig).to be_a_kind_of Tomte::Message
    expect(orig.foo).to eq :testingtest
  end


end
