require File.join(File.dirname(__FILE__), '../..', "tomte_protocol_spec_helper")

describe "A RPC response" do

  it 'should be able to transport hashes as response' do

    z = Tomte::Protocol::RPC::Response.new(:response => {:a => :foo, :b => :bar})
    t = z.to_tomte

    r = Tomte::Document::Parser.load(t)
    expect(r).to be_a_kind_of Tomte::Protocol::RPC::Response
    expect(r.response).to be_a_kind_of Hash
  end

  it 'should be able to transport arrays as response' do

    z = Tomte::Protocol::RPC::Response.new(:response => [:a, :b, :c])
    t = z.to_tomte

    r = Tomte::Document::Parser.load(t)
    expect(r).to be_a_kind_of Tomte::Protocol::RPC::Response
    expect(r.response).to be_a_kind_of Array
  end

  it 'should be able to transport objects inside arrays as response' do
    z = Tomte::Protocol::RPC::Response.new(:response => [Tomte::OpenDocument.new(:a => 1)])
    t = z.to_tomte

    r = Tomte::Document::Parser.load(t)
    expect(r).to be_a_kind_of Tomte::Protocol::RPC::Response
    expect(r.response).to be_a_kind_of Array
    expect(r.response.first).to be_a_kind_of Tomte::OpenDocument
  end

  it 'should be able to transport objects inside hashes as response' do
    z = Tomte::Protocol::RPC::Response.new(:response => {'a' => Tomte::OpenDocument.new(:b => 1)})
    t = z.to_tomte

    r = Tomte::Document::Parser.load(t)
    expect(r).to be_a_kind_of Tomte::Protocol::RPC::Response
    expect(r.response).to be_a_kind_of Hash
    expect(r.response['a']).to be_a_kind_of Tomte::OpenDocument
  end

  it 'should be able to transport objects inside array in hashes as response' do
    z = Tomte::Protocol::RPC::Response.new(:response => {'a' => [Tomte::OpenDocument.new(:b => 1)]})
    t = z.to_tomte

    r = Tomte::Document::Parser.load(t)
    expect(r).to be_a_kind_of Tomte::Protocol::RPC::Response
    expect(r.response).to be_a_kind_of Hash
    expect(r.response['a']).to be_a_kind_of Array
    expect(r.response['a'].first).to be_a_kind_of Tomte::OpenDocument
  end

  it 'should be able to transport objects inside hashes in hashes as response' do
    z = Tomte::Protocol::RPC::Response.new(:response => {'a' => { 'z' => Tomte::OpenDocument.new(:b => 1)}})
    t = z.to_tomte

    r = Tomte::Document::Parser.load(t)
    expect(r).to be_a_kind_of Tomte::Protocol::RPC::Response
    expect(r.response).to be_a_kind_of Hash
    expect(r.response['a']).to be_a_kind_of Hash
    expect(r.response['a']['z']).to be_a_kind_of Tomte::OpenDocument
  end

end